import { kebabCase } from 'lodash-es';

import { t } from '@/i18n';
import { fileContentTypeRegExp } from '@/constants';

import { fetchWithTimeout } from './fetch';

import type { GlobalToken } from 'ant-design-vue/es/theme';
import type { ApiResponse } from '@/types';

export const request = async <T>(url: string, init: RequestInit = {}, timeout?: number): Promise<T> => {
  const headers: HeadersInit = {};

  if (typeof init?.body === 'string') {
    headers['Content-Type'] = 'application/json;charset=UTF-8';
  }

  // const { token, saveToken, resetToken } = useUserInfoStore();
  // const { resetDevInfo } = useDeviceInfoStore();

  // if (token) {
  //   Object.assign(headers, {
  //     [TOKEN_KEY]: token,
  //   });
  // }

  headers['Authorization'] = 'Bearer ' + import.meta.env.VITE_TEMP_TOKEN;

  Object.assign(headers, init?.headers);

  const res = await fetchWithTimeout(
    import.meta.env.VITE_BASE_API + url,
    {
      ...init,
      headers,
    },
    timeout,
  );

  if (res.status === 401) {
    // removeToken();
    // resetToken();
    // resetDevInfo();
    // location.replace('/login');

    throw new Error(t('common.reLogin'));
  }

  if (!res.ok || res.status !== 200) {
    const json = await res.json();
    const errMsg: string = json.msg || json.error || '';
    throw new Error(`HTTP Error: ${res.status} ${errMsg}`);
  }

  // if (res.url.includes('/Uboxlogin')) {
  //   const token = res.headers.get(TOKEN_KEY);

  //   if (token) {
  //     setToken(token);
  //     saveToken(token);
  //   }
  // }
  const contentType = res.headers.get('content-type');
  const isBlob = fileContentTypeRegExp.test(contentType || '');

  if (isBlob) {
    return (await res.blob()) as T;
  }

  const json: ApiResponse<T> = await res.json();
  const { data, code, msg } = json;

  if (code !== 0) {
    throw new Error(msg);
  }

  if (data === null || data === undefined) {
    throw new Error(t('common.emptyData'));
  }

  return data;
};

export const translateNavigation = (title?: string) => {
  return title ? t('navigation.' + title) : '';
};

/**
 * 根据 Antd 当前主题的 token 生成 css var
 */
export const generateThemeCSSVar = (token: GlobalToken, prefix: string): string => {
  const cssVar = Object.entries(token)
    .filter(([key]) => {
      return /^(color|line|link|motion|border|margin|font(?!fam)|padding(?!con)|screen((?!min|max).)*$|boxShadow(?!pop|dra|tab|car))/i.test(
        key,
      );
    })
    .map(([key, value]) => {
      return `--${prefix}-${kebabCase(key)}:${value}${/(size|width|radius|margin|screen|padding)/gi.test(key) ? 'px' : ''}`;
    });

  cssVar.push(...getColorGradientCSSVar(token.colorPrimary, `${prefix}-color-primary`));

  return `:root{${cssVar.join(';')}}`;
};

/**
 * 根据十六进制值颜色生成透明度为 5%～95% 对应颜色的 css var
 */
export const getColorGradientCSSVar = (color: string, prefix: string): string[] => {
  const rgb = hexToRgb(color);

  if (!rgb) {
    return [];
  }

  const cssVar: string[] = [];
  const { r, g, b } = rgb;

  for (let i = 5; i <= 95; i += 5) {
    cssVar.push(`--${prefix}-opacity-${i}:rgba(${r}, ${g}, ${b}, ${i / 100})`);
  }

  return cssVar;
};

export const hexToRgb = (hex: string): { r: number; g: number; b: number } | null => {
  const cleanedHex = hex.replace(/^#/, '');

  if (cleanedHex.length !== 3 && cleanedHex.length !== 6) {
    console.error('Invalid hex color value');
    return null;
  }

  let extendedHex = cleanedHex;

  // 将 3 位十六进制扩展为 6 位（例如 #abc -> #aabbcc）
  if (cleanedHex.length === 3) {
    extendedHex = cleanedHex
      .split('')
      .map((char) => char + char)
      .join('');
  }

  const r = parseInt(extendedHex.slice(0, 2), 16);
  const g = parseInt(extendedHex.slice(2, 4), 16);
  const b = parseInt(extendedHex.slice(4, 6), 16);

  return { r, g, b };
};

export const addUnit = (val: number, unit: string = 'px'): string => {
  return val + unit;
};

export const waitTime = async (time: number = 100) => {
  await new Promise((resolve) => {
    setTimeout(resolve, time);
  });
};

export const downloadBlob = (blob: Blob, name: string) => {
  const url = window.URL.createObjectURL(blob);
  const a = document.createElement('a');

  a.href = url;
  a.download = name;

  document.body.appendChild(a);
  a.click();

  document.body.removeChild(a);
  window.URL.revokeObjectURL(url);
};
